/**
 * Add compass on top of the viewer canvas.
 * @param { Viewer } viewer
 */
function addCompass(viewer, size = 50) {
  const canvas = viewer.canvas;
  const compass = document.createElement("div");
  compass.style.position = "absolute";
  compass.style.bottom = "0px";
  compass.style.right = "0px";
  compass.style.width = `${size + 15}px`;
  compass.style.height = `${size}px`;
  compass.style.margin = "5px";
  compass.style.pointerEvents = "none";
  compass.style.display = "flex";

  const needle = new Image(size, size);
  needle.src = "./assets/compass/compass.svg";
  needle.style.transformOrigin = "center center";

  const N = document.createElement("span");
  N.textContent = "N";
  N.style.fontFamily = "sans-serif";
  N.style.color = "white";

  compass.appendChild(N);
  compass.appendChild(needle);

  viewer.camera.on("update", () => {
    const radAngle = viewer.camera.getRotation();
    needle.style.transform = `rotate(${radAngle}rad)`;
  });

  canvas.parentElement.appendChild(compass);
}

export default addCompass;
