function configure(viewer) {
  viewer.picker.on("pick", ({ object, rightClick }) => {
    if (rightClick) {
      viewer.camera.fitView(object.geometry.bounds);
    } else {
      if (document.getElementById("removeObjects").checked) {
        viewer.scene.removeObject(object.id);
      } else {
        if (object.selected) {
          viewer.scene.deselectObjects([object.id]);
        } else {
          viewer.scene.selectObjects([object.id]);
        }
      }
    }
  });
  viewer.canvas.addEventListener("click", e => {
    const { offsetX, offsetY } = e;
    const { x, y } = viewer.camera.transform.applyInverse({
      x: offsetX * window.devicePixelRatio,
      y: offsetY * window.devicePixelRatio,
    });
    console.log(
      `Clicked at canvas x: ${offsetX}, world x : ${x}, canvas y : ${offsetY}, world y: ${y}.`
    );
  });
  viewer.picker.on("hover", ({ object }) =>
    viewer.scene.highlightObjects([object.id])
  );
  viewer.picker.on("hover-out", ({ object }) =>
    viewer.scene.unhighlightObjects([object.id])
  );
  viewer.canvas.addEventListener("mouseleave", () => {
    const highlightedObjectIds = viewer.scene.objects
      .filter(o => o.highlighted)
      .map(o => o.id);
    viewer.scene.unhighlightObjects(highlightedObjectIds);
  });
}

export default configure;
