/**
 * @param { Viewer } viewer
 */
function addDoorsPlugin(viewer, label = "IfcDoor") {
  const button = document.getElementById("doors");
  const doorsStore = new Map();

  viewer.scene.on("object-update", ({ object, property }) => {
    const doorObject = doorsStore.get(object.id);
    if (doorObject && property === "visible" && object.visible === false) {
      viewer.scene.removeObject(doorObject.object.id);
      doorsStore.delete(object.id);
    }
  });

  viewer.scene.on("object-removed", object => {
    const doorObject = doorsStore.get(object.id);
    if (doorObject) {
      viewer.scene.removeObject(doorObject.object.id);
      doorsStore.delete(object.id);
    }
  });

  button.onclick = () => {
    if (
      doorsStore.size > 0 &&
      Array.from(doorsStore.values()).every(
        doorObject => doorObject.parent.visible
      )
    ) {
      doorsStore.forEach(doorObject =>
        viewer.scene.removeObject(doorObject.object.id)
      );
      doorsStore.clear();
    } else {
      const doors = viewer.scene.objects.filter(
        object => object.visible && object.type === label
      );
      doors.forEach(door => {
        /** @type { Line } */
        const line = door.geometry.getShape(); // assuming the last shape is the good one :P
        const p1 = line.getPoint(3); // assuming the firsts points are the good ones :P
        const p2 = line.getPoint(4);

        if (!p1 || !p2) return;

        const { normalize, dot, length, rotateAround } = viewer.utils.vector2D;

        const vector = { x: p2.x - p1.x, y: p2.y - p1.y };
        let angle = Math.acos(dot(normalize(vector), { x: 1, y: 0 }));

        if (p2.y - p1.y < 0) {
          angle = -angle;
        }

        const p3 = rotateAround(p2, p1, Math.PI / 2);

        const line1 = [p1.x, p1.y, p2.x, p2.y];
        const line2 = [p3.x, p3.y, p1.x, p1.y];
        const arc = {
          type: "arc",
          x: p1.x,
          y: p1.y,
          radius: length(vector),
          startAngle: angle,
          endAngle: angle + Math.PI / 2,
          composite: true,
        };

        const doorObject = viewer.scene.addObject({
          zIndex: 3,
          area: 0,
          lineColor: 0xff0000,
          lineWidth: 2,
          texture: "solid",
          textureOpacity: 0.2,
          textureTint: 0xff0000,
          geometry: [line1, arc, line2],
        });

        doorsStore.set(door.id, { parent: door, object: doorObject });
      });
    }
  };
}

export default addDoorsPlugin;
