const settings = {
  gridBackgroundColor: null,
  gridLinesColor: null,
  gridLinesWidth: 1,
  gridSpacing: 10,
};

/**
 * @param { Viewer } viewer
 */
function displayGrid(viewer, customSettings = null) {
  if (customSettings) {
    Object.assign(settings, customSettings);
  }
  const ctx = viewer.renderer.visibleContext.ctx;
  if (settings.gridBackgroundColor) {
    ctx.fillStyle =
      "#" + Number(settings.gridBackgroundColor).toString(16).padStart(6, 0);
    ctx.fillRect(0, 0, ctx.canvas.width, ctx.canvas.height);
  }
  if (settings.gridLinesColor) {
    ctx.lineWidth = settings.gridLinesWidth ?? 1;
    ctx.strokeStyle =
      "#" + Number(settings.gridLinesColor).toString(16).padStart(6, 0);

    drawGridLines(viewer);
  }
}

/**
 * @param { Viewer } viewer
 */
function drawGridLines(viewer) {
  const ctx = viewer.renderer.visibleContext.ctx;
  const { x: zoomX, y: zoomY } = viewer.camera.getScale();

  const spacing = settings.gridSpacing;

  const verticalSpacing = zoomX * spacing;
  const horizontalSpacing = zoomY * spacing;

  const { width, height } = ctx.canvas;

  ctx.beginPath();
  // Vertical lines
  for (let x = verticalSpacing; x < width; x += verticalSpacing) {
    ctx.moveTo(x, 0);
    ctx.lineTo(x, height);
  }
  // Horizontal lines
  for (let y = horizontalSpacing; y < height; y += horizontalSpacing) {
    ctx.moveTo(0, y);
    ctx.lineTo(width, y);
  }
  ctx.stroke();
}

export default displayGrid;
