/* eslint-disable no-unused-vars */
import makeViewer from "../dist/viewer2D.esm.js";
import displayFPS from "./fps.js";
import configure from "./configure.js";
import loadSimpleModel from "./loadSimpleModel.js";
import addCompass from "./compass.js";
import loadModelFromSVG from "./loadSVG.js";
import displayGrid from "./grid.js";
import loadTextures from "./textures.js";
import addDoorsPlugin from "./doors.js";
import addTextex from "./texter.js";
import addScaleWidget from "./scale.js";

const canvas = document.getElementById("canvas2d");
/** @type { Viewer } */
const viewer = makeViewer({ canvas });

const SCALE = 100;
const UNITS_SHOWN = 60;

loadTextures(viewer).then(() => {
  // loadSimpleModel(viewer);
  loadModelFromSVG(viewer, SCALE);
});

configure(viewer);

/*******  Blueprint mode :D *******/
const customSettings = {
  gridBackgroundColor: 0x294e9a,
  gridLinesColor: 0x4473d2,
  gridSpacing: UNITS_SHOWN,
};

const drawGrid = () => displayGrid(viewer, customSettings);
viewer.renderer.on("pre-draw", drawGrid);
viewer.settings.on("scale-update", drawGrid);

displayFPS(viewer);

addCompass(viewer);

addScaleWidget(viewer, SCALE, UNITS_SHOWN);

addDoorsPlugin(viewer);

addTextex(viewer);

window.viewer = viewer;
