import displayStoreySelector from "./storeySelector.js";

async function loadSvgModel(svgUri, viewer, scale) {
  const res = await fetch(svgUri);
  const svgString = await res.text();
  const svg = new DOMParser().parseFromString(svgString, "image/svg+xml");

  return viewer.utils.modelParsers.parseSvg(svg, { scale });
}

/**
 * @param { Viewer } viewer
 */
function loadModelFromSVG(viewer, scale = 1) {
  viewer.scene.textureManager.textureMatrix.rotate(30);
  /**********************************/

  loadSvgModel("./assets/models/MAP.svg", viewer, scale).then(model => {
    model.objects.forEach(object => {
      object.lineColor = 0xffffff;
      object.lineWidth = 2;
      if (object.type === "IfcWallStandardCase") {
        object.texture = "crossHatch1";
        object.lineWidth = 6;
      } else if (object.type === "IfcDoor") {
        object.texture = "solid";
        object.textureTint = 0xff0000;
        object.textureOpacity = 1;
        object.lineColor = 0x00ffff;
      }
    });

    const addedModel = viewer.scene.addModel(model);

    viewer.camera.fitView(addedModel.bounds);

    displayStoreySelector(addedModel, viewer);
  });
}

export default loadModelFromSVG;
