/**
 * @param { Viewer } viewer
 */
function addScaleWidget(
  viewer,
  offsetScale = 1,
  unitsShown = 100,
  unit = "meter",
  color = "white"
) {
  const { setText, setWidth } = addElements(viewer, color, offsetScale);

  viewer.camera.on("update", () => {
    let scale = viewer.camera.getScale().x;

    let unitOffset = 0;

    while (scale >= 1) {
      scale /= 1000;
      unitOffset++; // "mm", "m", "km"
    }

    const unitOffsets = ["k", "", "m"];

    const [length, mantissa] = getScaleBarLength(scale);

    setText(
      `${length} ${unitOffsets[unitOffset]}${unit}${length > 1 ? "s" : ""}` // TODO add offsetScale
    );
    setWidth(mantissa * unitsShown); // TODO fix unitsShown not correct! The width is correct but not the text.
  });

  window.getScaleBarLength = getScaleBarLength;
}

function getScaleBarLength(scale) {
  let mantissa = 1 / scale;
  let factor = 0;

  while (mantissa >= 1) {
    mantissa /= 10;
    factor++;
  }
  let mantissaMax, mantissaMin;

  const ranges = [1, 0.5, 0.3, 0.2, 0.15, 0.1]; // it assumes scale must be in ]0, 1].

  for (let i = 0; i < ranges.length; i++) {
    const rangeMax = ranges[i - 1] ?? Infinity;
    const rangeMin = ranges[i];

    if (mantissa >= rangeMin) {
      mantissaMax = rangeMax;
      mantissaMin = rangeMin;
      break;
    }
  }

  let closestMantissa = mantissaMax;
  if (mantissaMax / mantissa >= mantissa / mantissaMin) {
    closestMantissa = mantissaMin;
  }
  mantissa = closestMantissa / mantissa;
  factor = Math.pow(10, factor) * closestMantissa;

  return [factor, mantissa];
}

function addElements(viewer, color, offsetScale) {
  const canvas = viewer.canvas;

  const container = document.createElement("div");
  container.style.position = "absolute";
  container.style.bottom = "0px";
  container.style.left = "0px";
  container.style.height = "30px";
  // container.style.pointerEvents = "none";
  container.style.margin = "5px";
  container.style.display = "flex";
  container.style.flexDirection = "column";
  container.style.justifyContent = "space-between";

  const scale = document.createElement("div");
  scale.style.display = "inline-block";
  scale.style.position = "relative";
  scale.style.width = `${offsetScale * viewer.camera.getScale().x}px`;

  const horizontalBar = document.createElement("div");
  horizontalBar.style.height = "3px";
  horizontalBar.style.backgroundColor = color;

  const verticalBars = document.createElement("div");
  verticalBars.style.display = "flex";
  verticalBars.style.justifyContent = "space-between";

  const leftBar = document.createElement("div");
  leftBar.style.height = "6px";
  leftBar.style.width = "3px";
  leftBar.style.backgroundColor = color;

  const rightBar = document.createElement("div");
  rightBar.style.height = "6px";
  rightBar.style.width = "3px";
  rightBar.style.backgroundColor = color;

  verticalBars.appendChild(leftBar);
  verticalBars.appendChild(rightBar);

  scale.appendChild(verticalBars);
  scale.appendChild(horizontalBar);

  const setWidth = width => {
    scale.style.width = `${width}px`;
  };

  const text = document.createElement("div");
  text.style.color = color;
  const textNode = document.createTextNode("");
  text.appendChild(textNode);

  const setText = text => {
    textNode.nodeValue = text;
  };

  container.appendChild(text);
  container.appendChild(scale);

  canvas.parentElement.appendChild(container);

  return { setWidth, setText };
}

export { addScaleWidget as default };
